/*
 * vim: shiftwidth=4 tabstop=4 expandtab cindent:
 */

const urlTime = '/data/time';
const urlMessage = '/generated/message.txt';
const urlEvent = '/data/event';
const urlSchedule = '/data/schedule';
const urlCompData = '/data/comp/{id}';
const urlAthletes = '/api/v1/event/{eid}/athletes';
const urlAthlete = '/api/v1/event/{eid}/athlete/{aid}/comps';
const classActiveNav = 'active';

const COMP_STATE_PLANNED = 0;
const COMP_STATE_SCHEDULED = 1;
const COMP_STATE_STARTED = 2;
const COMP_STATE_FALSESTART = 3;
const COMP_STATE_OFFICIAL = 4;
const COMP_STATE_FINISHED = 5;
const COMP_STATE_FOTOFINISH = 6;
const COMP_STATE_CANCELLED = 7;
const COMP_STATE_GETTING_READY = 8;
const COMP_STATE_DRAFT = 9;

const roundLUT = {
    'R': 'Hauptrennen',
    'V': 'Vorlauf',
    'H': 'Hoffnungslauf',
    'Q': 'Viertelfinale',
    'S': 'Halbfinale',
    'F': 'Finale',
    'L': 'Rangliste',
    'A': 'Abteilung',
    'T': 'Testrennen',
    'Z': 'Zwischenlauf'
};

const compStateLUT = {
    [COMP_STATE_PLANNED]: 'ohne Daten',
    [COMP_STATE_SCHEDULED]: 'geplant',
    [COMP_STATE_STARTED]: 'gestartet',
    [COMP_STATE_FALSESTART]: 'Fehlstart',
    [COMP_STATE_OFFICIAL]: 'offiziell',
    [COMP_STATE_FINISHED]: 'beendet',
    [COMP_STATE_FOTOFINISH]: 'Fotofinish',
    [COMP_STATE_CANCELLED]: 'gestrichen',
    [COMP_STATE_GETTING_READY]: 'in Vorbereitung',
    [COMP_STATE_DRAFT]: 'Entwurf'
};

const VIEW_TYPE_RESULTS = 'results';
const VIEW_TYPE_ATHLETES = 'athletes';
const VIEW_TYPE_ATHLETE = 'athlete';
const VIEW_TYPE_STARTLIST = 'startlist';
const VIEW_TYPE_SCHEDULE = 'schedule';

var viewState = {
    compIdx: -1,
    athlete: null,
    view: VIEW_TYPE_SCHEDULE,
    compModeView: VIEW_TYPE_STARTLIST,
};

var data = {
    eventInfo: null,
    loading: false,
    pendingUpdate: null,
    athletes: {
        list: null,
        current: null
    },
    comps: {
        list: [],
        lut: [],
        current: null
    }
};

var indicator = null;
var wsNotify = null;

function onTimeReceive(transport)
{
    var time = transport.responseText.evalJSON();

    $('clock').innerHTML = time.h + ':' + (time.m < 10 ? '0' : '') + time.m + " Uhr";
    $('clock').classList.remove('error');
}

function onTimeComplete(request)
{
    if (request.readyState == 4 && request.status == 0) {
        $('clock').classList.add('error');
    }
}

function onMessageReceive(response)
{
    if (response.status === 200 && response.responseText.length > 0) {
        $('message').show().innerHTML = response.responseText;
    } else {
        $('message').hide();
    }
}

function onEventReceive(transport)
{
    const eventInfo = transport.responseText.evalJSON()[0];

    $('event').innerHTML = eventInfo.title + ', ' + eventInfo.venue;

    if (!data.eventInfo || eventInfo.id != data.eventInfo.id) {
        data.eventInfo = eventInfo;
        data.comps.current = null;
        fetchSchedule();
    }
}


//-------------------------------------------------------------------
// Schedule

async function fetchSchedule()
{
    $$('li.comp_display').each(function(i) { i.hide(); });
    $('comp').hide();
    indicator.show();

    try {
        const response = await fetch(urlSchedule);
        if (!response.ok) {
            throw new Error(`Response status: ${response.status}`);
        }

        onScheduleReceive(await response.json());
    } catch (err) {
        console.log(`Error fetching schedule ${err.message}`);
    }
}

function scheduleColumnCallback(row, rowData)
{
    const comp_dt = new Date(rowData.datetime);

    row.insert(new Element('td').update(rowData.isDummy ? '' : rowData.no))
    row.insert(new Element('td').update(rowData.isDummy ? '' : comp_dt.toLocaleDateString(undefined, { weekday: 'short', month: 'short', day: 'numeric' })));
    row.insert(new Element('td').update(rowData.isDummy ? '' : comp_dt.toLocaleTimeString(undefined, { timeStyle: 'short' })));
    row.insert(new Element('td').update(rowData.isDummy ? rowData.Label : rowData.RaceNumber).writeAttribute('colspan', rowData.isDummy ? 5 : 1))

    if (!rowData.isDummy) {
        row.insert(new Element('td').update(rowData.RaceShortLabel + (rowData.RaceComment ? ' ' + rowData.RaceComment : '')));
        row.insert(new Element('td').update( roundLUT[rowData.RoundCode] ? roundLUT[rowData.RoundCode] + ' ' + rowData.Label : ''));

        if (!rowData.isCancelled) {
            row.insert(new Element('td').update(compStateLUT[rowData.State]));
            //row.insert(new Element('td', { 'class': 'center'}).update(new Element('a', { href: '#' }).update('>>') ));
        } else {
            row.insert(new Element('td', { colSpan: 3 }).update('gestrichen'));
        }

        row.addEventListener('click', (e) => {
            viewState.view = viewState.compModeView;
            fetchComp(rowData.compID, true);
        } );
        row.addClassName('clickable');
        if (rowData.State === COMP_STATE_STARTED) {
            // THINK: highlight running comps (running, false start, finished)
            row.addClassName(`state${rowData.State}`);
        }
    } else {
        row.addClassName('schedule-dummy');
    }

    return true;
}

function onScheduleReceive(o)
{
    data.comps.list = o.list;
    data.comps.list.forEach((comp) => { if (!data.isDummy) { data.comps.lut[`c${comp.compID}`] = comp; } } );
    data.comps.current = null;

    highlightNavLi($('menu_schedule'));
    createTableFromJson(['LaufNr', 'Tag', 'Startzeit', 'RNR', 'Rennen', 'Lauf', 'Status'], data.comps.list, scheduleColumnCallback);
    indicator.hide();


    checkAndShowScrollbar();
}

//-------------------------------------------------------------------
// competition data receive

function buildAthleteElements(a, showClub, isLast, isCox)
{
    retval = new Element('div', { 'class': 'athlete' + (isCox ? ' cox' : '') + ((a && a.isTainted) ? ' tainted' : '') });
    /* TODO: make links (but we have to have the athlete list at this point) */

    if (isCox) {
        retval.insert(new Element('span', { 'class': 'cox_indicator' }).update('St. '));
    }

    if (!a) {
        retval.insert(new Element('span', { 'class': 'missing' }).update('N.N.'));
        return retval;
    }

    retval.insert(
        new Element('span', { 'class': 'name' }).update(a.firstName + ' ')
    ).insert(
        new Element('span', { 'class': 'lastName' }).update(a.lastName + ' ')
    ).insert(
        new Element('span', { 'class': 'yob' }).update('(' + a.yob + (showClub ? ',&nbsp;' + a.clubUltraAbbr : '') + ')' + (!isLast ? ',' : ''))
    );

   return retval;
}

function buildCrewElements(startlist, entry, compInfo, root)
{
    crew = {};
    startlist.athletes.findAll(function(a) { return (a.crewID == entry.ID) && (a.Round <= compInfo.Round); }).each(function(a) {
        var isTainted = crew[a.pos] !== undefined;
        crew[a.pos] = a;
        crew[a.pos].isTainted = isTainted;
    });

    const numPositions = compInfo.numRowers + compInfo.isCoxed;
    for (var i = 1; i <= numPositions; ++i) {
        root.appendChild(buildAthleteElements(crew[i], entry.isTeam, i === numPositions, i > compInfo.numRowers));
    }

    return root;
}

function getResultStr(r, withBreak)
{
    if (r) {
        var c = '<span class="bold">' + (r.displayValue || r.display) + '</span>';
        if ((r.splitNr || r.split_number) != 64) {
            c = c + '&nbsp;(' + r.rank + ')'
        }
        if ((r.displayType == 'T' || r.delta > 0) && r.rank > 1) {
            c = c + (withBreak ? '<br/>' : '&nbsp;(') + '+';
            m = Math.floor(r.delta / (1000 * 60));
            s = Math.floor((r.delta / 1000) % 60);
            z = Math.floor((r.delta % 1000) / 10);
            if (m > 0) {
                c = c + m + ':' + ((s < 10) ? '0' : '');
            }
            c = c + s + '.' + ((z < 10) ? '0' : '') + z;
            c = c + (!withBreak ? ')' : '');
        }

        return c;
    } else {
        return '-';
    }
}

function compColumnCallback(row, rowData, startlist)
{
    var i, a = '';
    var comp = startlist.props;

    if (rowData.has_yellow_card) {
        row.classList.add('yellow_card');
    }

    if (viewState.view === VIEW_TYPE_RESULTS) {
        row.insert(new Element('td', { 'class': 'rank' }).update(rowData.rank > 0 ? rowData.rank : ''));
    }

    row.insert(new Element('td', { 'class': 'lane' }).update(rowData.lane));
    row.insert(new Element('td', { 'class': 'bib' }).update(rowData.full_bib));
    row.insert(
        new Element('td').insert(
            new Element('div', { 'class': 'boatLabel' }).update(rowData.isTeam ? rowData.shortLabel : rowData.longLabel)
        ).insert(
            new Element('div', { 'class': 'boatComment' }).update(rowData.comment)
        ).insert(
            buildCrewElements(startlist, rowData, comp, new Element('div', { 'class': 'crew' }))
        )
    );

    if (viewState.view === VIEW_TYPE_RESULTS) {
        for (i = 1; i <= comp.numSplits + 0; ++i) {
            sid = 's' + i;
            row.insert(new Element('td', { 'class': 'result' }).update(getResultStr(rowData.results[sid], true)));
        }

        var fr = rowData.results['s64'];
        row.insert(new Element('td', { 'class': 'result' }).update(comp.State == COMP_STATE_OFFICIAL || (fr && fr.rtype != 'N') ? getResultStr(fr, true) : '...'));
    }

    return true;
}

function buildCompUI()
{
    const c = data.comps.current;
    const comp = c.props;
    var headers = ['Bahn', 'StNr', 'Boot/Mannschaft'];
    var i, header;

    /* sort comp data and insert headers for result (if displayed) */
    if (viewState.view === VIEW_TYPE_RESULTS) {
        headers.unshift('Rang');
        for (i = 1; i <= comp.numSplits + 1; ++i) {
            headers.push((comp.distance / (comp.numSplits + 1) * i) + '&nbsp;m');
        }
        c.entries = c.entries.sortBy(function(e) { return e.resultSortValue; });
    } else {
        c.entries = c.entries.sortBy(function(e) { return e.lane; });
    }

    const comp_dt = new Date(comp.datetime)
    header = new Element('div', {'class': 'compHeader'}).insert(
        new Element('div', { 'class': 'left' }).insert(
            new Element('div', { 'class': 'bold' }).update('Re.&nbsp' + comp.RaceNumber + ', ' + comp.RaceShortLabel + (comp.RaceComment ? '&nbsp;'+ comp.RaceComment : '')+ ', ' + roundLUT[comp.RoundCode] + ' ' + comp.Label)
        ).insert(
            new Element('div', { 'class': 'left' }).update('Lauf ' + comp.no + ', Start: ' +
                (new Intl.DateTimeFormat(undefined, { dateStyle: 'long' }).format(comp_dt)) + ', ' +
                (new Intl.DateTimeFormat(undefined, { timeStyle: 'short' }).format(comp_dt)))
        )
    ).insert(
        new Element('div', { 'class': 'right state state' + comp.State + (comp.isCancelled ? ' cancelled ' : ' ')}).update(comp.isCancelled ? 'gestrichen' : compStateLUT[comp.State])
    ).insert(
        new Element('div', { 'class': 'break' })
    );

    createTableFromJson(headers, c.entries, compColumnCallback, { head: header, userData: c });

    if (c.referees.length > 0 && viewState.view === VIEW_TYPE_RESULTS) {
        const refBlock = new Element('div', { 'class': 'referees' }).insert(new Element('h4').update('Schiedsrichter'));
        const list = new Element('ul');
        c.referees.forEach((r) => {
            list.insert((new Element('li')).update(`${r.Referee_LastName}, ${r.Referee_FirstName} (${r.Referee_City})`))
        });

        refBlock.insert(list);
        $('data').insert(refBlock);
    }

    if (comp.ruleText) {
        const progDiv = new Element('div', { 'class': 'progressionRule' }).
            insert(new Element('span', { 'class': 'label' }).update('Qualifikation: ')).
            insert(new Element('span', { 'class': 'ruleText' }).update(comp.ruleText));
        $('data').insert(progDiv);
    }

    if (c.withdrawals.length > 0) {
        let wdBlock = new Element('div', { 'class': 'withdrawals' }).insert(new Element('h4').update('Abmeldungen in Rennen ' + comp.RaceNumber));
        let list = new Element('ul');
        for (i = 0; i < c.withdrawals.length; ++i) {
            const li = new Element('li');
            li.insert(new Element('span', {'class': 'bib'}).update(c.withdrawals[i].bib));
            li.insert(new Element('span', {'class': 'label'}).update(c.withdrawals[i].label));
            list.insert(li);
        }

        wdBlock.insert(list);
        $('data').insert(wdBlock);
    }

    checkAndShowScrollbar();
}

function highlightCompViewState()
{
    if (viewState.view === VIEW_TYPE_RESULTS) {
        highlightNavLi($('menu_result'));
    } else if (viewState.view === VIEW_TYPE_STARTLIST) {
        highlightNavLi($('menu_startlist'));
    }
}


function onCompReceive(o)
{
    if (o) {
        data.comps.current = o;
        data.comps.current.props = o.comp[0];
        data.comps.current.comp = null;
        /* assign result sort value and split results to boat */
        data.comps.current.entries.each(function(boat) {
                boat.results = {};
                splits = data.comps.current.results.filter(function(s) { return boat.ID == s.ID; } );
                splits.each(function(s) { boat.results['s' + s.splitNr] = s });
                if (boat.results.s64) {
                    boat.resultSortValue = boat.results.s64.sortValue;
                    boat.rank = boat.results.s64.rank;
                } else {
                    boat.resultSortValue = 99999999 + boat.lane;
                    boat.rank = null;
                }
        });

        highlightCompViewState();
        updateViewStateCompIdx();

        $$('li.comp_display').each(function(i) { i.show(); });
        $('comp').show();

        buildCompUI();
    }

    indicator.hide();
    data.loading = false;
}

function updateViewStateCompIdx()
{
    /* set navigation to correct index */
    for (i = 0; i < data.comps.list.length; ++i) {
        if (data.comps.list[i].compID == data.comps.current.props.compID) {
            if (viewState.compIdx !== i) {
                viewState.compIdx = i;
            }
            break;
        }
    }
}

async function fetchComp(compId, updateHistory)
{
    if (data.loading) {
        return;
    }

    cancelDeferredCompUpdate();
    data.loading = true;
    indicator.show();

    try {
        const response = await fetch(urlCompData.replace('{id}', compId));
        if (!response.ok) {
            throw new Error(`Response status: ${response.status}`);
        }

        const json = await response.json();
        onCompReceive(json);

        if (updateHistory) {
            pushViewStateToHistory();
        }
    } catch (err) {
        console.log(err.message);
    }
}


function createTableFromJson(header, body, columnCallback, options)
{
    const table = new Element('table', { 'class': 'data' });
    var tr = new Element('tr');
    const tbody = new Element('tbody');
    var i, length = body.length;

    header.each(function(e) { tr.insert(new Element('th').update(e)) });

    table.insert(new Element('thead').insert(tr));

    for(i = 0; i < length; ++i) {
        tr = new Element('tr');
        if (columnCallback(tr, body[i], (options && options.userData) ? options.userData : null)) {
            tbody.insert(tr);
        }
    }

    table.insert(tbody);

    $('data').childElements().each(function(e){e.remove()});

    if (options && options.head) {
        $('data').insert(options.head);
    }

    $('data').insert(table);

    if (options && options.foot) {
        $('data').insert(options.foot);
    }
}

//-------------------------------------------------------------------
// current comp stuff

// get the most current comp:
//   1. the first started / false start comp
//   2. the first scheduled
//   3. the last unofficial/official/fotofinish comp
//   4. the first comp in schedule (whatever state it has)
function findCurrentComp(schedule)
{
    schedule = schedule.filter(function(c) { return (!c.isCancelled) });

    var comps = schedule.filter(function (c) { return (c.State == COMP_STATE_STARTED || c.State == COMP_STATE_FALSESTART) });
    if (comps.length > 0) { return comps[0]; }

    comps = schedule.filter(function (c) { return (c.State == COMP_STATE_SCHEDULED) });
    if (comps.length > 0) { return comps[0]; }

    comps = schedule.filter(function (c) { return (c.State == COMP_STATE_OFFICIAL || c.State == COMP_STATE_FINISHED || c.State == COMP_STATE_FOTOFINISH) });
    if (comps.length > 0) { return comps[comps.length - 1]; }

    if (schedule.length > 0) { return schedule[0]; }

    return null;
}

function findAndFetchCurrentComp(schedule)
{
    const c = findCurrentComp(schedule);

    if (c) {
        fetchComp(c.compID, true);
    }
}

async function onCurrentCompClick()
{
    const r = await fetch(urlSchedule);
    data.comps.list = (await r.json()).list;
    findAndFetchCurrentComp(data.comps.list);
}

//-------------------------------------------------------------------
// athletes

async function fetchAthletes(forceReload)
{
    highlightNavLi(document.getElementById('menu_athletes'));

    /* cacheing disabled for the time being */
    if (true || !data.athletes.list || forceReload) {
        const r = await fetch(urlAthletes.replace('{eid}', data.eventInfo.id));
        data.athletes.list = await r.json();
    }

    createTableFromJson(['Nachname', 'Vorname', 'Jahrgang', 'Verein'], data.athletes.list, (row, athlete) => {
        row.insert(new Element('td').update(athlete.lastname));
        row.insert(new Element('td').update(athlete.firstname));
        row.insert(new Element('td').update(athlete.yob + ' ' + (athlete.sex == 'W' ? '♀' : '♂')));
        row.insert(new Element('td').update(athlete.club_name));

        row.classList.add('clickable');
        row.addEventListener('click', (e) => {
            document.getElementById('menu_athlete').show();
            viewState.view = VIEW_TYPE_ATHLETE;
            viewState.athlete = athlete;
            pushViewStateToHistory();
            fetchAthlete();

            e.stop();
        })

        return true;
    });
}

async function fetchAthlete()
{
    const athlete = viewState.athlete;
    const navItem = document.getElementById('menu_athlete');
    highlightNavLi(navItem);
    navItem.firstChild.innerHTML = `${athlete.lastname}, ${athlete.firstname}`;

    var athleteComps = [];
    try {
        const r = await fetch(urlAthlete.replace('{eid}', data.eventInfo.id).replace('{aid}', athlete.id));
        if (r.ok) {
            athleteComps = await r.json();
        }
    } catch (err) {
        console.log(`Error fetching athlete: ${err.message}`);
    }

    createTableFromJson(['LaufNr', 'Tag', 'Startzeit', 'RNR', 'Rennen', 'Lauf', 'Status', 'Rang', 'Ergebnis', 'Qualifikation'], athleteComps, (row, item) => {
        scheduleColumnCallback(row, data.comps.lut[`c${item.Comp.id}`]);

        row.insert(new Element('td').update(item.Result ? item.Result.rank : '-' ));
        row.insert(new Element('td').update(item.Result ? getResultStr(item.Result, false) : '-' ));
        row.insert(new Element('td').update(item.Comp.ruleText));

        return true;
    });
}

//-------------------------------------------------------------------
// navigation

function onNextCompClick()
{
    for (let i = viewState.compIdx + 1; i < data.comps.list.length; ++i) {
        if (!data.comps.list[i].isDummy) {
            fetchComp(data.comps.list[i].compID, true);
            break;
        }
    }
}

function onPrevCompClick()
{
    for (let i = viewState.compIdx - 1; i >= 0; --i) {
        if (!data.comps.list[i].isDummy) {
            fetchComp(data.comps.list[i].compID, true);
            break;
        }
    }
}

function onCurrentCompReloadClick()
{
    fetchComp(data.comps.current.props.compID, false);
}

function highlightNavLi(e)
{
    $$('nav#main_nav ul li').each(function(e){ e.removeClassName(classActiveNav); });
    e.addClassName(classActiveNav);
}

function checkAndShowScrollbar()
{
    $('scrollbar').toggle(document.body.scrollHeight > document.documentElement.clientHeight);
}

function switchCompDisplayMode(newView)
{
    if (newView === viewState.view) {
        return;
    }

    viewState.view = newView;
    viewState.compModeView = newView;

    highlightCompViewState();
    buildCompUI();

    pushViewStateToHistory();
}

function cancelDeferredCompUpdate()
{
    if (data.pendingUpdate) {
        clearTimeout(data.pendingUpdate);
        data.pendingUpdate = null;
    }
}

function deferredCompUpdate(id)
{
    // defer update for at most 125 ms
    const timeout = Math.random() * 125;
    cancelDeferredCompUpdate();
    data.pendingUpdate = window.setTimeout(fetchComp, timeout, id);
}

function onNotificationReceived(e)
{
    var msg = JSON.parse(e.data);

    if (!msg || !msg.type) {
        return;
    }

    if (viewState.view === VIEW_TYPE_SCHEDULE) {
        if (msg.type == 'schedule:change') {
            fetchSchedule();
            return;
        }
    } else if (data.comps.current && msg.type.startsWith('comp:')) {
        /* a comp is viewed */
        if (data.comps.current.props.compID != msg.compID) {
            return;
        }

        if (msg.type === 'comp:stateChange') {
            fetchComp(msg.compID, false);
        } else if (msg.type === 'comp:startlistChange' || msg.type === 'comp:resultChange') {
            deferredCompUpdate(msg.compID);
        }
    }
}

// ------------------------------------------------------------------
// history

function pushViewStateToHistory()
{
    updateDocumentTitle();
    history.pushState(viewState, '');
}

function updateDocumentTitle()
{
    if (viewState.view === VIEW_TYPE_SCHEDULE) {
        document.title = `Zeitplan ${data.eventInfo.title}`;
    } else if (viewState.view === VIEW_TYPE_ATHLETES) {
        document.title = `Athleten`;
    } else if (viewState.view === VIEW_TYPE_ATHLETE) {
        document.title = `${viewState.athlete.lastname}, ${viewState.athlete.firstname}`;
    } else if (viewState.view === VIEW_TYPE_STARTLIST || viewState.view === VIEW_TYPE_RESULTS) {
        const comp = data.comps.list[viewState.compIdx];
        const title = (viewState.view === VIEW_TYPE_STARTLIST ? 'Startliste' : 'Ergebnis') +
            ' Re. ' + comp.RaceNumber +
            ' / ' + comp.RaceShortLabel +
            ' / ' + roundLUT[comp.RoundCode] +
            ' ' + comp.Label + ` (#${comp.no})`;

        document.title = title;
    }
}

function onHistoryStatePopped(e)
{
    const oldState = e.state;

    if (oldState === null) {
        return;
    }

    viewState = e.state;
    if (viewState.view === VIEW_TYPE_SCHEDULE) {
        fetchSchedule();
    } else if (viewState.view === VIEW_TYPE_ATHLETES) {
        fetchAthletes();
    } else if (viewState.view === VIEW_TYPE_ATHLETE) {
        fetchAthlete();
    } else if (viewState.view === VIEW_TYPE_STARTLIST || viewState.view === VIEW_TYPE_RESULTS) {
        fetchComp(data.comps.list[oldState.compIdx].compID, false);
    }

    updateDocumentTitle();
}

// ------------------------------------------------------------------
// setup
function initInfoPoint()
{
    new Ajax.Request(urlTime, {method: 'get', onSuccess: onTimeReceive.bind(self)});
    new Ajax.Request(urlMessage, {method: 'get', onComplete: onMessageReceive.bind(self)});
    new Ajax.Request(urlEvent, {method: 'get', onSuccess: onEventReceive.bind(self)});

    new PeriodicalExecuter(() => { new Ajax.Request(urlTime, {method: 'get', onSuccess: onTimeReceive.bind(this), onComplete: onTimeComplete.bind(this) }) }, 5);
    new PeriodicalExecuter(() => { new Ajax.Request(urlMessage, {method: 'get', onComplete: onMessageReceive.bind(this)}) }, 30);
    new PeriodicalExecuter(() => { new Ajax.Request(urlEvent, {method: 'get', onSuccess: onEventReceive.bind(this)}) }, 60);

    window.onpopstate = onHistoryStatePopped;
    document.addEventListener('beforeunload', () => { if (wsNotify) { wsNotify.close(); } });

    $('nav_top').observe('click', (e) => { window.scrollTo({ top: 0, left: 0, behavior: 'smooth' }); e.stop(); });
    $('nav_up').observe('click', (e) => { window.scrollBy({ top: -300, left: 0, behavior: 'smooth' }); e.stop(); });
    $('nav_down').observe('click', (e) => { window.scrollBy({ top: +300, left: 0, behavior: 'smooth' }); e.stop(); });
    $('nav_bottom').observe('click', (e) => { window.scrollTo({ top: document.body.scrollHeight, left: 0, behavior: 'smooth' }); e.stop(); });

    $('nav_next').observe('click', (e) => { onNextCompClick(); e.stop(); });
    $('nav_current').observe('click', (e) => { onCurrentCompClick(); e.stop(); });
    $('nav_reload').observe('click', (e) => { onCurrentCompReloadClick(); e.stop(); });
    $('nav_prev').observe('click', (e) => { onPrevCompClick(); e.stop(); });

    document.getElementById('menu_schedule').addEventListener('click', (e) => {
        /* reset comp navigation */
        if (viewState.view !== VIEW_TYPE_SCHEDULE) {
            viewState.view = VIEW_TYPE_SCHEDULE;
            pushViewStateToHistory();
        }
        fetchSchedule();
        e.stop();
    });

    document.getElementById('menu_athletes').addEventListener('click', (e) => {
        if (viewState.view !== VIEW_TYPE_ATHLETES) {
            viewState.view = VIEW_TYPE_ATHLETES;
            pushViewStateToHistory();
        }
        fetchAthletes();
        e.stop();
    });

    document.getElementById('menu_athlete').addEventListener('click', (e) => {
        fetchAthlete();
        e.stop();
    });

    document.getElementById('menu_result').addEventListener('click', (e) => {
        switchCompDisplayMode(VIEW_TYPE_RESULTS);
        e.stop();
    });

    document.getElementById('menu_startlist').addEventListener('click', (e) => {
        switchCompDisplayMode(VIEW_TYPE_STARTLIST);
        e.stop();
    });

    indicator = document.getElementById('indicator');
    indicator.hide();

    wsNotify = new WebSocket('ws://' + location.host + '/notify/', 'aqntfy');
    wsNotify.onmessage = onNotificationReceived;

    history.replaceState(viewState, '');
}

// ------------------------------------------------------------------
// main
document.addEventListener('DOMContentLoaded', () => { initInfoPoint(); });
